{#
  Copyright DB InfraGO AG and contributors
  SPDX-License-Identifier: Apache-2.0
#}
{% from 'common_macros.html.j2' import show_other_attributes %}
{% from 'functional.html.j2' import describe_function %}

{{ describe_function(object) | safe }}

<h2>Other properties of "{{ object.name }}"</h2>
{%-
  set excluded = [
    "applied_property_value_groups",
    "available_in_states",
    "context_diagram",
    "description",
    "inputs",
    "name",
    "outputs",
    "owner",
    "parent",
    "property_value_groups",
    "realization_view",
    "related_exchanges",
    "validate",
    "xtype",
  ]
-%}
{{ show_other_attributes(object, excluded=excluded) | safe }}
