{#
  Copyright DB InfraGO AG and contributors
  SPDX-License-Identifier: Apache-2.0
#}
{% from 'common_macros.html.j2' import show_other_attributes, linked_name, display_property_values %}

<h1>Physical Link <b>{{ object.name }}</b></h1>

{% if object.parent %}
  <p>A part of {{ linked_name(object.parent) | safe }}</p>
{% endif %}

{{ render_diagram(object, "context_diagram") }}

<p>
  Connects {{ linked_name(object.ends[0].owner) | safe }} via
  {{ linked_name(object.ends[0]) | safe }} port and
  {{ linked_name(object.ends[1].owner) | safe }} via
  {{ linked_name(object.ends[1]) | safe }} port.
</p>

{{ display_property_values(object ) | safe }}
{%-
  set excluded = [
    "applied_property_value_groups",
    "context_diagram",
    "ends",
    "linkEnds",
    "name",
    "owner",
    "parent",
    "property_value_groups",
    "source",
    "target",
    "xtype",
  ]
-%}
{{ show_other_attributes(object, excluded=excluded) | safe }}
